package services

import (
	"encoding/json"
	"errors"
	"fmt"
	"hongze/hongze_mfyx/models"
	"hongze/hongze_mfyx/utils"
	"io/ioutil"
	"net/http"
	"strconv"
	"strings"
	"time"
)

type SendWxCategoryTemplate struct {
	WxAppId        string   `description:"公众号appId"`
	Keywords       []string `description:"模板消息keyword字段"`
	TemplateId     string   `description:"模板id"`
	RedirectUrl    string   `description:"跳转地址"`
	RedirectTarget int      `description:"小程序跳转目标:1:弘则研报小程序,2:随手办公小程序,3:查研观向"`
	Resource       string   `description:"资源唯一标识"`
	SendType       int      `description:"发送的消息类型:1:报告,2:指标更新提醒,3:审批通知,4:销售领取客户通知,5:活动取消通知,6活动更改时间通知,7:关注的作者发布报告通知,8:发送日报（周报、双周报、月报）模板消息,9:活动预约/报名时间通知"`
	OpenIdArr      []string `description:"消息接收者openid"`
}

// 推送类目模板消息
func SendCategoryTemplateMsg(sendInfo *SendWxCategoryTemplate) (err error) {
	postData, err := json.Marshal(sendInfo)
	if err != nil {
		go utils.SendAlarmMsg("SendCategoryTemplateMsg json.Marshal Err:"+err.Error(), 1)
		return err
	}
	body := ioutil.NopCloser(strings.NewReader(string(postData)))
	client := &http.Client{}
	req, err := http.NewRequest("POST", utils.SendWxCategoryTemplateMsgUrl, body)
	if err != nil {
		go utils.SendAlarmMsg("SendCategoryTemplateMsg http.NewRequest Err:"+err.Error(), 1)
		return err
	}
	contentType := "application/json;charset=utf-8"
	req.Header.Set("Content-Type", contentType)
	req.Header.Set("Authorization", utils.SendTemplateMsgAuthorization)
	resp, err := client.Do(req)
	if err != nil {
		fmt.Println("http client.Do Err:" + err.Error())
		return err
	}
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		return err
	}
	result := new(models.BaseResponse)
	err = json.Unmarshal(b, &result)
	if err != nil {
		return err
	}
	if result.Ret != 200 {
		err = errors.New(string(b))
		return err
	}
	return
}

// 发送用户操作活动消息提醒
func SendWxCategoryMsgWithActivityUserRemind(keyWord1, keyWord2, keyWord3, keyWord4 string, openIdList []*models.OpenIdList, activityId int) (err error) {
	if len(openIdList) == 0 {
		return
	}
	var msg string
	defer func() {
		if err != nil {
			go utils.SendEmail("发送模版消息失败"+"【"+utils.APPNAME+"】"+time.Now().Format("2006-01-02 15:04:05"), msg+";Err:"+err.Error(), utils.EmailSendToUsers)
			go utils.SendAlarmMsg(fmt.Sprint("发送用户操作活动消息提醒,文章ID：", activityId), 2)
			utils.FileLog.Info("发送模版消息失败,Err:%s", err.Error())
		}
	}()
	keyWord1 = utils.TruncateActivityNameString(keyWord1)
	accessToken, err := models.GetWxAccessTokenByXzs()
	if err != nil {
		msg = "GetWxAccessToken Err:" + err.Error()
		return
	}
	if accessToken == "" {
		msg = "accessToken is empty"
		return
	}
	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	var keywords []string
	keywords = append(keywords, keyWord1, keyWord2, keyWord3, keyWord4)

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxMsgCategoryTemplateIdActivityPublishApply
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_APPLY
	sendInfo.OpenIdArr = openIdArr
	err = SendCategoryTemplateMsg(sendInfo)

	//sendMap["miniprogram"] = map[string]interface{}{"appid": utils.WxAppId, "pagepath": "activityPages/activityDetail/activityDetail?id=" + strconv.Itoa(activityId)}
	return
}

func SendPermissionApplyCategoryTemplateMsg(realName, companyName, mobile, applyMethod string, itemOpenid *models.OpenIdList) (err error) {

	defer func() {
		if err != nil {
			go utils.SendAlarmMsg(fmt.Sprint("权限申请模板消息推送消息失败,手机号", mobile, "公司：", companyName, "手机号：", mobile, ";Err:"+err.Error()), 2)
		}
	}()
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	var keyword5 string
	var redirectUrl string

	detail, e := models.GetCygxApplyRecordByMobile(mobile)
	if e != nil {
		err = errors.New("GetActivitySignupNomeetingCountList" + e.Error())
		return
	}
	//如果是潜在客户就推送可查看详情的模版消息
	if detail.ApplyMethod == 2 {
		redirectUrl = utils.WX_MSG_PATH_APPLY_DETAIL + strconv.Itoa(detail.ApplyRecordId)
	}
	keyword1 = "权限申请：" + realName + "【" + companyName + "】"
	keyword2 = mobile
	keyword3 = time.Now().Format(utils.FormatDateTime)
	keyword4 = applyMethod
	openIdArr := make([]string, 0)
	openIdArr = append(openIdArr, itemOpenid.OpenId)
	sendInfo := new(SendWxCategoryTemplate)

	keyword1 = realName
	keyword2 = companyName
	keyword3 = "权限申请"
	keyword4 = time.Now().Format(utils.FormatDateTimeMinute2)
	keyword5 = applyMethod

	sendInfo.Keywords = []string{keyword1, keyword2, keyword3, keyword4, keyword5}
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdApplyXzs
	sendInfo.RedirectUrl = redirectUrl
	sendInfo.RedirectTarget = 3
	sendInfo.Resource = mobile
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_APPLY
	sendInfo.OpenIdArr = openIdArr
	err = SendCategoryTemplateMsg(sendInfo)
	return
}

// 发送用户操作活动消息提醒
func SendNeiRongZuActivitieSignCategoryTemplateMsg(user *models.WxUserItem, activityDetail *models.ActivityDetail) (err error) {
	var msg string
	defer func() {
		if err != nil {
			go utils.SendEmail("发送模版消息失败"+"【"+utils.APPNAME+"】"+time.Now().Format("2006-01-02 15:04:05"), msg+";Err:"+err.Error(), utils.EmailSendToUsers)
			go utils.SendAlarmMsg(fmt.Sprint("发送用户操作活动消息提醒,文章ID：", activityDetail.ActivityId), 2)
			utils.FileLog.Info("发送模版消息失败,Err:%s", err.Error())
		}
	}()
	accessToken, err := models.GetWxAccessTokenByXzs()
	if err != nil {
		msg = "GetWxAccessToken Err:" + err.Error()
		return
	}
	if accessToken == "" {
		msg = "accessToken is empty"
		return
	}
	sellerItemQy, err := models.GetSellerByCompanyIdCheckFicc(user.CompanyId, 2)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	configCode := utils.TPL_MSG_NEI_RONG_ZU
	cnf, e := models.GetConfigByCode(configCode)
	if e != nil {
		err = errors.New("GetConfigByCode, Err: " + e.Error() + configCode)
		return
	}
	openIdList, e := models.GetUserRecordListByMobile(4, cnf.ConfigValue+","+sellerItemQy.Mobile+utils.MobileShenTao)
	if e != nil && e.Error() != utils.ErrNoRow() {
		err = errors.New("GetUserRecordListByMobile, Err: " + e.Error() + cnf.ConfigValue)
		return
	}

	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	var keywords []string
	keywords = append(keywords, user.RealName, user.CompanyName,
		sellerItemQy.RealName, time.Now().Format(utils.FormatDateTimeMinute2),
		"报名："+activityDetail.ActivityName)

	//sendMap["miniprogram"] = map[string]interface{}{"appid": utils.WxAppId, "pagepath": "activityPages/activityDetail/activityDetail?id=" + strconv.Itoa(activityDetail.ActivityId)}
	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdApplyXzs
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_APPLY
	sendInfo.OpenIdArr = openIdArr
	err = SendCategoryTemplateMsg(sendInfo)
	return
}

func SendActivitieCancelSignCategoryTemplateMsg(user *models.WxUserItem, activityDetail *models.ActivityDetail) (err error) {
	var msg string
	defer func() {
		if err != nil {
			go utils.SendEmail("发送模版消息失败"+"【"+utils.APPNAME+"】"+time.Now().Format("2006-01-02 15:04:05"), msg+";Err:"+err.Error(), utils.EmailSendToUsers)
			go utils.SendAlarmMsg(fmt.Sprint("发送用户操作活动消息提醒,文章ID：", activityDetail.ActivityId), 2)
			utils.FileLog.Info("发送模版消息失败,Err:%s", err.Error())
		}
	}()
	accessToken, err := models.GetWxAccessTokenByXzs()
	if err != nil {
		msg = "GetWxAccessToken Err:" + err.Error()
		return
	}
	if accessToken == "" {
		msg = "accessToken is empty"
		return
	}
	sellerItemQy, err := models.GetSellerByCompanyIdCheckFicc(user.CompanyId, 2)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	configCode := utils.TPL_MSG_NEI_RONG_ZU
	cnf, e := models.GetConfigByCode(configCode)
	if e != nil {
		err = errors.New("GetConfigByCode, Err: " + e.Error() + configCode)
		return
	}
	openIdList, e := models.GetUserRecordListByMobile(4, cnf.ConfigValue+","+sellerItemQy.Mobile+utils.MobileShenTao)
	if e != nil && e.Error() != utils.ErrNoRow() {
		err = errors.New("GetUserRecordListByMobile, Err: " + e.Error() + cnf.ConfigValue)
		return
	}

	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}

	var keywords []string
	keywords = append(keywords, user.RealName, user.CompanyName,
		sellerItemQy.RealName, time.Now().Format(utils.FormatDateTimeMinute2),
		"取消报名"+"【"+activityDetail.ActivityName+"】")

	//sendMap["miniprogram"] = map[string]interface{}{"appid": utils.WxAppId}

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.WxAppId = utils.WxAppId
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdApplyXzs
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_APPLY
	sendInfo.OpenIdArr = openIdArr
	err = SendCategoryTemplateMsg(sendInfo)
	return
}

// 发送用户留言提醒
func SendCommentWxCategoryTemplateMsg(req models.AddCygxArticleCommentReq, user *models.WxUserItem, articleInfo *models.ArticleDetail, msgId int) (err error) {
	var msg string
	defer func() {
		if err != nil {
			go utils.SendEmail("发送类目模版消息失败"+"【"+utils.APPNAME+"】"+time.Now().Format("2006-01-02 15:04:05"), msg+";Err:"+err.Error(), utils.EmailSendToUsers)
			go utils.SendAlarmMsg(fmt.Sprint("发送用户用户留言提醒失败,文章ID：", req.ArticleId), 2)
			utils.FileLog.Info("发送类目模版消息失败,Err:%s", err.Error())
		}
	}()

	companyItem, e := models.GetSellerDetailAllByCompanyId(user.CompanyId)
	if e != nil {
		return
	}
	if companyItem == nil {
		msg = "获取客户所属销售信息失败,UserID：" + strconv.Itoa(user.UserId)
	}
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	var keyword5 string
	var redirectUrl string
	var mobile string

	var keywords []string
	keyword1 = user.RealName
	keyword2 = companyItem.CompanyName
	keyword3 = companyItem.SellerName
	keyword4 = time.Now().Format(utils.FormatDateTimeMinute2)
	keyword5 = "报告留言：" + req.Content
	keyword5 = utils.TruncateActivityNameString(keyword5)
	keywords = append(keywords, keyword1, keyword2, keyword3, keyword4, keyword5)

	if articleInfo.ArticleTypeId > 0 {
		//研选报告添加汪洋
		mobile = companyItem.Mobile + "," + utils.WxMsgTemplateIdAskMsgMobilePublic + "," + utils.ActSendMsgMobile
	} else {
		mobile = companyItem.Mobile + "," + utils.WxMsgTemplateIdAskMsgMobilePublic
	}
	openIdList, e := models.GetWxOpenIdByMobileList(mobile)
	if e != nil {
		err = errors.New("GetWxOpenIdByMobileList, Err: " + e.Error())
		return
	}
	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	//mapSource留言跳转类型，1:文章、2:活动、3:微路演视频、4:活动视频、5:活动音频
	if utils.RunMode != "release" {
		redirectUrl = utils.WX_MSG_PATH_ARTICLE_DETAIL + strconv.Itoa(articleInfo.ArticleId)
	} else {
		redirectUrl = utils.WX_MSG_PATH_TEMPLATE_MSG + "SourceId=" + strconv.Itoa(msgId) + "&SourceType=1"
	}

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdApplyXzs
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_APPLY
	sendInfo.OpenIdArr = openIdArr
	sendInfo.RedirectUrl = redirectUrl
	err = SendCategoryTemplateMsg(sendInfo)
	return
}

// 发送报告提问消息提醒
func SendWxCategoryMsgWithAsk(companyName, realName, sellerName, askTime, askMsg string, openIdList []*models.OpenIdList, articleId int) (err error) {
	if len(openIdList) == 0 {
		return
	}
	var msg string
	defer func() {
		if err != nil {
			go utils.SendEmail("发送类目模版消息失败"+"【"+utils.APPNAME+"】"+time.Now().Format("2006-01-02 15:04:05"), msg+";Err:"+err.Error(), utils.EmailSendToUsers)
			go utils.SendAlarmMsg(fmt.Sprint("发送报告提问消息提醒类目模版消息失败,用户名", realName, "文章ID：", articleId), 2)
			utils.FileLog.Info("发送类目模版消息失败,Err:%s", err.Error())
		}
		fmt.Println("line 21", err, msg)
	}()
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	var keyword5 string

	keyword1 = realName
	keyword2 = companyName
	keyword3 = sellerName
	keyword4 = askTime
	keyword4 = "活动提问：" + askMsg

	keywords := []string{keyword1, keyword2, keyword3, keyword4, keyword5}

	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	redirectUrl := utils.WX_MSG_PATH_ARTICLE_DETAIL + strconv.Itoa(articleId)

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdApplyXzs
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
	sendInfo.OpenIdArr = openIdArr
	sendInfo.RedirectUrl = redirectUrl
	err = SendCategoryTemplateMsg(sendInfo)
	return
}

// SendActivitieSignCategoryTemplateMsg 线下调研活动扫码签到给对应销售发类目模班消息
func SendActivitieSignCategoryTemplateMsg(user *models.WxUserItem, activityDetail *models.ActivityDetail) (err error) {
	defer func() {
		if err != nil {
			go utils.SendAlarmMsg(fmt.Sprint("线下调研活动扫码签到给对应销售发模班消息推送失败Err:", err.Error(), "活动ID：", activityDetail.ActivityId, "手机号：", user.Mobile), 2)
		}
	}()
	var mobiles []string
	//获取销售信息
	sellerItem, err := models.GetSellerByCompanyIdCheckFicc(user.CompanyId, 2)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return
	}
	if sellerItem != nil {
		mobiles = append(mobiles, sellerItem.Mobile, utils.MobileWuAngDi, utils.MobileDongHeng, utils.MobileShenTao)
	}

	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	var keyword5 string

	var redirectUrl string

	keyword1 = user.RealName
	keyword2 = user.CompanyName
	keyword3 = sellerItem.RealName
	keyword4 = activityDetail.ActivityName
	keyword5 = time.Now().Format(utils.FormatDateTimeMinute2)

	openIdList, e := models.GetWxOpenIdByMobileSliceList(mobiles)
	if e != nil {
		err = errors.New("GetWxOpenIdByMobileList, Err: " + e.Error())
		return
	}
	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	if utils.RunMode != "release" {
		redirectUrl = utils.WX_MSG_PATH_ACTIVITY_DETAIL + strconv.Itoa(activityDetail.ActivityId)
	} else {
		redirectUrl = utils.WX_MSG_PATH_ACTIVITY_SIGN + strconv.Itoa(activityDetail.ActivityId)
	}

	keywords := []string{keyword1, keyword2, keyword3, keyword4, keyword5}
	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdActivitySign
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
	sendInfo.OpenIdArr = openIdArr
	sendInfo.RedirectUrl = redirectUrl
	err = SendCategoryTemplateMsg(sendInfo)
	if err != nil {
		return
	}
	return
}

// SendReviewCategoryTemplateMsgAdmin 提交审核时给王芳，汪洋发消息
func SendReviewCategoryTemplateMsgAdmin(specialId int) (err error) {
	defer func() {
		if err != nil {
			go utils.SendAlarmMsg(fmt.Sprint("处理试用申请给王芳，汪洋发消息失败, specialId:", specialId, "ErrMsg", err.Error()), 2)
		}
	}()
	var configCode string
	//如果是研选的就推送给汪洋跟王芳，否则就推送给王芳
	configCode = utils.TPL_MSG_WANG_FANG_WANG_YANG
	cnf, e := models.GetConfigByCode(configCode)
	if e != nil {
		err = errors.New("GetConfigByCode, Err: " + e.Error() + configCode)
		return
	}
	openIdList, e := models.GetUserRecordListByMobile(4, cnf.ConfigValue)
	if e != nil && e.Error() != utils.ErrNoRow() {
		err = errors.New("GetUserRecordListByMobile, Err: " + e.Error() + cnf.ConfigValue)
		return err
	}
	specialItem, e := models.GetYanxuanSpecialItemById(specialId)
	if e != nil {
		err = errors.New("GetYanxuanSpecialFollowUserById, Err: " + e.Error())
		return
	}
	user, e := models.GetWxUserItemByUserId(specialItem.UserId)
	if e != nil {
		err = errors.New("GetWxUserItemByUserId, Err: " + e.Error())
		return err
	}
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	keyword1 = specialItem.NickName
	keyword2 = specialItem.RealName + "-" + user.CompanyName
	keyword3 = specialItem.SpecialName
	keyword3 = time.Now().Format(utils.FormatDateTimeMinute2)
	keyword4 = "研选专栏提交了内容待审核"
	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	redirectUrl := ""
	redirectUrl = utils.WX_MSG_PATH_YX_SPECIAL_ENABLE_DETAIL + strconv.Itoa(specialId)

	keywords := []string{keyword1, keyword2, keyword3, keyword4}

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxCategoryMsgTemplateIdApplyXzs
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
	sendInfo.OpenIdArr = openIdArr
	sendInfo.RedirectUrl = redirectUrl
	err = SendCategoryTemplateMsg(sendInfo)
	if err != nil {
		return
	}
	return
}

// 研选专栏审核完成时，给提交人发送类目模板消息
func SendWxCategoryMsgSpecialAuthor(specialId, status int) (err error) {
	defer func() {
		if err != nil {
			go utils.SendAlarmMsg(fmt.Sprint("研选专栏审核完成时，给提交人发送模板消息", specialId, ", specialId", err.Error()), 2)
		}
	}()
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	var redirectUrl string

	specialItem, e := models.GetYanxuanSpecialItemById(specialId)
	if e != nil {
		err = errors.New("GetYanxuanSpecialFollowUserById, Err: " + e.Error())
		return
	}
	user, e := models.GetWxUserItemByUserId(specialItem.UserId)
	if e != nil {
		err = errors.New("GetWxUserItemByUserId, Err: " + e.Error())
		return err
	}

	openIdList, err := models.GetUserRecordListByMobile(4, user.Mobile)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return err
	}

	keyword1 = specialItem.Title
	keyword2 = specialItem.NickName
	if status == 1 {
		keyword3 = "文章已通过审核，点击查看详情"
		redirectUrl = utils.WX_MSG_PATH_YX_SPECIAL_DETAIL + strconv.Itoa(specialId)
	} else {
		keyword3 = "文章未通过审核，点击查看驳回原因"
		redirectUrl = utils.WX_MSG_PATH_YX_SPECIAL_CENTER
	}
	keyword4 = time.Now().Format(utils.FormatDateTimeMinute2)
	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}

	keywords := []string{keyword1, keyword2, keyword3, keyword4}

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxMsgCategoryTemplateIdCompanyApprovalMessageRai
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
	sendInfo.OpenIdArr = openIdArr
	sendInfo.RedirectUrl = redirectUrl
	err = SendCategoryTemplateMsg(sendInfo)
	if err != nil {
		return
	}
	return
}

// 研选专栏有新内容审核通过时，给关注此专栏的客户发送模板消息
func SendWxCategoryMsgSpecialFollow(specialId int) (err error) {
	defer func() {
		if err != nil {
			go utils.SendAlarmMsg(fmt.Sprint("研选专栏有新内容审核通过时，给关注此专栏的客户发送类目模板消息失败", specialId, ", specialId", err.Error()), 2)
		}
	}()
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string

	followers, e := models.GetYanxuanSpecialFollowUserById(specialId)
	if e != nil {
		err = errors.New("GetYanxuanSpecialFollowUserById, Err: " + e.Error())
		return
	}
	if len(followers) == 0 {
		return
	}
	specialItem, e := models.GetYanxuanSpecialItemById(specialId)
	if e != nil {
		err = errors.New("GetYanxuanSpecialFollowUserById, Err: " + e.Error())
		return
	}
	var allInUserId string
	for _, v := range followers {
		allInUserId += strconv.Itoa(v) + ","
	}

	allInUserId = strings.TrimRight(allInUserId, ",")

	userList, err := models.GetWxUserListByUserIds(allInUserId)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return err
	}

	mobile := ``

	for _, v := range userList {
		mobile += v.Mobile + ","
	}

	mobile = strings.TrimRight(mobile, ",")

	openIdList, e := models.GetWxOpenIdByMobileList(mobile)
	if e != nil {
		err = errors.New("GetSellerByAdminId, Err: " + e.Error())
		return
	}
	if len(openIdList) == 0 {
		return
	}

	keyword1 = specialItem.SpecialName
	keyword2 = "发布了新报告"
	keyword3 = specialItem.Title
	keyword4 = time.Now().Format(utils.FormatDateTimeMinute2)
	openIdArr := make([]string, 0)
	for _, v := range openIdList {
		openIdArr = append(openIdArr, v.OpenId)
	}
	redirectUrl := ""
	redirectUrl = utils.WX_MSG_PATH_YX_SPECIAL_DETAIL + strconv.Itoa(specialId)
	keywords := []string{keyword1, keyword2, keyword3, keyword4}

	sendInfo := new(SendWxCategoryTemplate)
	sendInfo.Keywords = keywords
	sendInfo.TemplateId = utils.WxMsgCategoryTemplateIdCompanyApprovalMessageRai
	sendInfo.RedirectTarget = 3
	sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
	sendInfo.OpenIdArr = openIdArr
	sendInfo.RedirectUrl = redirectUrl
	err = SendCategoryTemplateMsg(sendInfo)
	if err != nil {
		return
	}
	return
}

// 文章互动相关类目模版消息
func SendWxCategoryMsgInteractive(user *models.WxUserItem, interactive string, articleId int) (err error) {
	//30秒内阅读同一篇报告不做重复推送
	key := "CYGX_" + utils.CYGX_OBJ_RESEARCHSUMMARY + "_READ" + strconv.Itoa(articleId) + "_" + strconv.Itoa(user.UserId)
	if utils.Rc.IsExist(key) {
		return
	}
	utils.Rc.Put(key, 1, 30*time.Second)
	defer func() {
		if err != nil {
			go utils.SendAlarmMsg(fmt.Sprint("文章互动相关，类目模板消息推送 ,SendWxCategoryMsgInteractive"+err.Error(), "userId：", user.UserId, "互动类型:", interactive), 2)
		}
	}()
	countUser, err := models.GetUserRemind(user.UserId)
	if err != nil {
		return err
	}
	if countUser == 0 {
		return err
	}
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	//获取销售手机号
	sellerItemQy, err := models.GetSellerByCompanyIdCheckFicc(user.CompanyId, 2)
	if err != nil && err.Error() != utils.ErrNoRow() {
		return err
	}
	if sellerItemQy != nil {
		openIdList, e := models.GetWxOpenIdByMobileList(sellerItemQy.Mobile)
		if e != nil {
			err = errors.New("GetSellerByAdminId, Err: " + e.Error())
			return
		}
		if len(openIdList) == 0 {
			return
		}
		detail, e := models.GetCygxResearchSummaryInfoById(articleId)
		if e != nil {
			err = errors.New("GetCygxProductInteriorDetail, Err: " + e.Error())
			return
		}
		if detail == nil {
			return
		}
		keyword1 = user.RealName + "-" + user.CompanyName
		keyword1 = utils.TruncateActivityNameString(keyword1)
		keyword2 = fmt.Sprint("互动提醒：", interactive)
		keyword3 = detail.Title
		keyword4 = time.Now().Format(utils.FormatDateTimeMinute2)
		openIdArr := make([]string, 0)
		for _, v := range openIdList {
			openIdArr = append(openIdArr, v.OpenId)
		}

		redirectUrl := ``
		redirectUrl = utils.WX_MSG_PATH_ARTICLE_DETAIL + strconv.Itoa(articleId)

		keywords := []string{keyword1, keyword2, keyword3, keyword4}

		sendInfo := new(SendWxCategoryTemplate)
		sendInfo.Keywords = keywords
		sendInfo.TemplateId = utils.WxMsgCategoryTemplateIdActivityPublishApply
		sendInfo.RedirectTarget = 3
		sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
		sendInfo.OpenIdArr = openIdArr
		sendInfo.RedirectUrl = redirectUrl
		err = SendCategoryTemplateMsg(sendInfo)
		if err != nil {
			return
		}
	}
	return
}

// 用户搜索操作操作行为，类目模板消息推送
func SearchKeywordUserRmindCategoryMsg(user *models.WxUserItem, keyWord string) (err error) {
	defer func() {
		if err != nil {
			fmt.Println(err)
			go utils.SendAlarmMsg(fmt.Sprint("用户搜索操作操作行为，类目模板消息推送失败"+err.Error(), "UserId:", user.UserId, "keyWord:", keyWord), 2)
		}
	}()
	countUser, e := models.GetUserRemind(user.UserId)
	if e != nil {
		err = errors.New("GetUserRemind, Err: " + e.Error())
		return
	}
	if countUser == 0 {
		return
	}
	var keyword1 string
	var keyword2 string
	var keyword3 string
	var keyword4 string
	//获取销售手机号
	sellerItemQy, e := models.GetSellerByCompanyIdCheckFicc(user.CompanyId, 2)
	if e != nil && e.Error() != utils.ErrNoRow() {
		err = errors.New("GetSellerByCompanyIdCheckFicc, Err: " + e.Error())
		return
	}
	if sellerItemQy != nil {
		openIdList, e := models.GetWxOpenIdByMobileList(sellerItemQy.Mobile)
		if e != nil {
			err = errors.New("GetSellerByAdminId, Err: " + e.Error())
			return
		}
		if len(openIdList) == 0 {
			return
		}
		keyword1 = user.RealName+ "-"+ user.CompanyName
		keyword2 = fmt.Sprint("互动提醒：搜索" )
		keyword3 = keyWord
		keyword4 = time.Now().Format(utils.FormatDateTimeMinute2)
		openIdArr := make([]string, 0)
		for _, v := range openIdList {
			openIdArr = append(openIdArr, v.OpenId)
		}
		redirectUrl := ""
		keywords := []string{keyword1, keyword2, keyword3, keyword4}

		sendInfo := new(SendWxCategoryTemplate)
		sendInfo.Keywords = keywords
		sendInfo.TemplateId = utils.WxMsgCategoryTemplateIdCompanyApprovalMessageRai
		sendInfo.RedirectTarget = 3
		sendInfo.SendType = utils.TEMPLATE_MSG_CYGX_ARTICLE_ADD
		sendInfo.OpenIdArr = openIdArr
		sendInfo.RedirectUrl = redirectUrl
		err = SendCategoryTemplateMsg(sendInfo)
		if err != nil {
			return
		}
	}
	return
}
